@@ -1,20 +1,18 @@
 package Mason::Util;
-BEGIN {
-  $Mason::Util::VERSION = '2.06';
-}
 use Carp;
 use Class::MOP;
 use Class::Unload;
 use Fcntl qw( :DEFAULT :seek );
 use File::Find;
 use File::Spec::Functions ();
+use JSON;
 use Try::Tiny;
 use strict;
 use warnings;
 use base qw(Exporter);
 
 our @EXPORT_OK =
-  qw(can_load catdir catfile checksum combine_similar_paths dump_one_line find_wanted first_index is_absolute mason_canon_path read_file touch_file trim uniq write_file);
+  qw(can_load catdir catfile checksum combine_similar_paths dump_one_line find_wanted first_index is_absolute json_encode json_decode mason_canon_path read_file touch_file trim uniq write_file);
 
 my $Fetch_Flags          = O_RDONLY | O_BINARY;
 my $Store_Flags          = O_WRONLY | O_CREAT | O_BINARY;
@@ -146,6 +144,18 @@ sub is_absolute {
     return substr( $path, 0, 1 ) eq '/';
 }
 
+# Maintain compatibility with both JSON 1 and 2. Borrowed from Data::Serializer::JSON.
+#
+sub json_decode {
+    my ($text) = @_;
+    return JSON->VERSION < 2 ? JSON->new->jsonToObj($text) : JSON->new->decode($text);
+}
+
+sub json_encode {
+    my ($data) = @_;
+    return JSON->VERSION < 2 ? JSON->new->objToJson($data) : JSON->new->utf8->encode($data);
+}
+
 sub mason_canon_path {
 
     # Like File::Spec::canonpath but with a few fixes.
